/*
 * Copyright 2022, TeamDev. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Redistribution and use in source and/or binary forms, with or without
 * modification, must retain the above copyright notice and the following
 * disclaimer.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package io.spine.validate.option;

import com.google.auto.service.AutoService;
import com.google.common.collect.ImmutableSet;
import com.google.errorprone.annotations.Immutable;
import io.spine.annotation.Internal;

import java.util.Set;

import static com.google.common.collect.Sets.union;

/**
 * A factory of standard validating options for non-primitive types.
 */
@AutoService(ValidatingOptionFactory.class)
@Internal
@Immutable
public final class NonPrimitiveOptionFactory implements StandardOptionFactory {

    private static final ImmutableSet<FieldValidatingOption<?>> STRING_OPTIONS =
            ImmutableSet.of(Pattern.create());
    private static final ImmutableSet<FieldValidatingOption<?>> COLLECTION_OPTIONS =
            ImmutableSet.of(Required.create(false), Goes.create(), Distinct.create());
    private static final ImmutableSet<FieldValidatingOption<?>> MESSAGE_OPTIONS =
            ImmutableSet.of(new Valid());

    @Override
    public Set<FieldValidatingOption<?>> forString() {
        return union(STRING_OPTIONS, COLLECTION_OPTIONS);
    }

    @Override
    public Set<FieldValidatingOption<?>> forByteString() {
        return COLLECTION_OPTIONS;
    }

    @Override
    public Set<FieldValidatingOption<?>> forEnum() {
        return COLLECTION_OPTIONS;
    }

    @Override
    public Set<FieldValidatingOption<?>> forMessage() {
        return union(MESSAGE_OPTIONS, COLLECTION_OPTIONS);
    }
}
